package jrummikub.control;

import jrummikub.model.GameSettings;
import jrummikub.model.GameState;
import jrummikub.model.IRoundState;
import jrummikub.model.RoundState;
import jrummikub.util.IListener;
import jrummikub.view.IView;

/**
 * Controls a Game, at some point including all Rounds, starts new Rounds
 */
public class GameControl {
	private GameSettings gameSettings;
	private IView view;
	private RoundControl roundControl;
	private GameState gameState;

	/**
	 * Constructor
	 * 
	 * @param gameSettings
	 *          the game settings
	 * @param view
	 *          the view
	 */
	public GameControl(GameSettings gameSettings, IView view) {
		this.gameSettings = gameSettings;
		this.view = view;
		
		gameState = new GameState();
		gameState.setFirstRoundFirstPlayer((int)(Math.random() * gameSettings.getPlayerList().size()));

		view.getNewGameEvent().add(new IListener() {
			@Override
			public void handle() {
				startRound();
			}
		});
		view.getQuitEvent().add(new IListener() {
			@Override
			public void handle() {
				quitProgram();
			}
		});
	}

	/**
	 * Game gets started by initializing the first Round
	 */
	public void startGame() {
		startRound();
	}

	private void startRound() {
		if (roundControl != null) {
			return;
		}

		IRoundState roundState = new RoundState(gameSettings);

		// TODO: add number of already played rounds
		roundState.setActivePlayerNumber(gameState.getFirstRoundFirstPlayer());
		
		roundControl = new RoundControl(roundState, view);
		roundControl.getEndRoundEvent().add(new IListener() {

			@Override
			public void handle() {
				roundControl = null;
			}
		});

		roundControl.startRound();
	}

	private void quitProgram() {
		System.exit(0);
	}

}
